@@ -15,6 +15,8 @@
  *  WWW at http://www.gnu.org/copyleft/gpl.html).
  */
 
+void Rprintf(const char *, ...);
+
 #include <sys/types.h>
 
 #include "globals.h"
@@ -33,16 +35,23 @@
  * @param bf        Buffer in which to set up the opened BF
  * @return          1 on success, 0 on failure (not like fopen/fclose)
  */
-int BFopen(char *filename, char *type, BFile *bf)
+int
+BFopen(char *filename, char *type, BFile *bf)
 {
-  bf->fd = fopen(filename, type);
-  bf->buf = '\0';
-  bf->bits_in_buf = 0;
+  /* force binary-mode open */
+  char passmode[3] = { '\0', 'b', '\0' };
+
   bf->mode = type[0];
-  bf->position = 0;
 
   assert((bf->mode == 'r') || (bf->mode == 'w'));
 
+  passmode[0] = bf->mode;
+
+  bf->fd = fopen(filename, passmode);
+  bf->buf = '\0';
+  bf->bits_in_buf = 0;
+  bf->position = 0;
+
   return (bf->fd ? 1 : 0);
 }
 
@@ -56,7 +65,8 @@ int BFopen(char *filename, char *type, BFile *bf)
  * @param bf    Buffer in which to set up the opened BS
  * @return      boolean: 1 on success, 0 on failure (not like fopen/fclose)
  */
-int BSopen(unsigned char *base, char *type, BStream *bf)
+int
+BSopen(unsigned char *base, char *type, BStream *bf)
 {
   bf->base = base;
   bf->buf = '\0';
@@ -75,9 +85,10 @@ int BSopen(unsigned char *base, char *type, BStream *bf)
  * If this is an output buffer, it is flushed before closing.
  *
  * @param stream  The file buffer to close.
- * @return        Always returns true.
+ * @return        Returns true iff the file was closed successfully.
  */
-int BFclose(BFile *stream)
+int
+BFclose(BFile *stream)
 {
   if (stream->mode == 'w') 
     BFflush(stream);
@@ -93,7 +104,8 @@ int BFclose(BFile *stream)
  * @param stream  The stream buffer to close.
  * @return        Always returns true.
  */
-int BSclose(BStream *stream)
+int
+BSclose(BStream *stream)
 {
   if (stream->mode == 'w') 
     BSflush(stream);
@@ -113,7 +125,8 @@ int BSclose(BStream *stream)
  * @param stream  The file buffer to flush.
  * @return        Boolean: 1 for all OK, 0 for a problem.
  */
-int BFflush(BFile *stream)
+int
+BFflush(BFile *stream)
 {
   int retval;
 
@@ -160,7 +173,8 @@ int BFflush(BFile *stream)
  * @param stream  The stream to flush.
  * @return        Boolean: 1 for all OK, 0 for a problem.
  */
-int BSflush(BStream *stream)
+int
+BSflush(BStream *stream)
 {
   int retval;
 
@@ -207,7 +221,8 @@ int BSflush(BStream *stream)
  * @param stream  The buffer to write via.
  * @return        Boolean: 1 for all OK, 0 for a problem.
  */
-int BFwrite(unsigned char data, int nbits, BFile *stream)
+int
+BFwrite(unsigned char data, int nbits, BFile *stream)
 {
 
   unsigned char mask;
@@ -251,7 +266,8 @@ int BFwrite(unsigned char data, int nbits, BFile *stream)
  * @param stream  The buffer to write via.
  * @return        Boolean: 1 for all OK, 0 for a problem.
  */
-int BSwrite(unsigned char data, int nbits, BStream *stream)
+int
+BSwrite(unsigned char data, int nbits, BStream *stream)
 {
   unsigned char mask;
 
@@ -293,7 +309,8 @@ int BSwrite(unsigned char data, int nbits, BStream *stream)
  * @param stream  The BFile buffer to use.
  * @return        Boolean: 1 for all OK, 0 for a problem.
  */
-int BFread(unsigned char *data, int nbits, BFile *stream)
+int
+BFread(unsigned char *data, int nbits, BFile *stream)
 {
   *data = '\0';
 
@@ -328,7 +345,8 @@ int BFread(unsigned char *data, int nbits, BFile *stream)
  * @param stream  The BStream buffer to use.
  * @return        Boolean: 1 for all OK, 0 for a problem.
  */
-int BSread(unsigned char *data, int nbits, BStream *stream)
+int
+BSread(unsigned char *data, int nbits, BStream *stream)
 {
   *data = '\0';
 
@@ -365,13 +383,14 @@ int BSread(unsigned char *data, int nbits, BStream *stream)
  * @param stream  The BFile buffer to use.
  * @return        Boolean: 1 for all OK, 0 for a problem.
  */
-int BFwriteWord(unsigned int data, int nbits, BFile *stream)
+int
+BFwriteWord(unsigned int data, int nbits, BFile *stream)
 {
   int bytes, rest, i;
   unsigned char *cdata;
 
   if ((nbits > 32) || (nbits < 0)) {
-    fprintf(stderr, "bitio.o/BFwriteWord: nbits (%d) not in legal bounds\n", nbits);
+    Rprintf("bitio.o/BFwriteWord: nbits (%d) not in legal bounds\n", nbits);
     return 0;
   }
 
@@ -406,13 +425,14 @@ int BFwriteWord(unsigned int data, int nbits, BFile *stream)
  * @param stream  The BFile buffer to use.
  * @return        Boolean: 1 for all OK, 0 for a problem.
  */
-int BFreadWord(unsigned int *data, int nbits, BFile *stream)
+int
+BFreadWord(unsigned int *data, int nbits, BFile *stream)
 {
   int bytes, rest, i;
   unsigned char *cdata;
 
   if ((nbits > 32) || (nbits < 0)) {
-    fprintf(stderr, "bitio.o/BFreadWord: nbits (%d) not in legal bounds\n", nbits);
+    Rprintf("bitio.o/BFreadWord: nbits (%d) not in legal bounds\n", nbits);
     return 0;
   }
 
@@ -444,7 +464,8 @@ int BFreadWord(unsigned int *data, int nbits, BFile *stream)
 /**
  * Gets the stream position of a BFile.
  */
-int BFposition(BFile *stream)
+int
+BFposition(BFile *stream)
 {
   assert(stream);
   return stream->position;
@@ -453,7 +474,8 @@ int BFposition(BFile *stream)
 /**
  * Gets the stream position of a BStream.
  */
-int BSposition(BStream *stream)
+int
+BSposition(BStream *stream)
 {
   assert(stream);
   return stream->position;
@@ -464,9 +486,10 @@ int BSposition(BStream *stream)
  *
  * @param stream  The stream whose position marker is changed.
  * @param offset  The desired new offset.
- * @return  Always true.
+ * @return        Always true.
  */
-int BSseek(BStream *stream, off_t offset)
+int
+BSseek(BStream *stream, off_t offset)
 {
   stream->buf = '\0';
   stream->bits_in_buf = 0;
