@@ -1,11 +1,11 @@
 library(RcppCWB)
 
-testthat::context("cwb_cpos2id")
+testthat::context("cl_cpos2id")
 
 test_that(
   "cpos2id",
   {
-    ids <- cwb_cpos2id(
+    ids <- cl_cpos2id(
       corpus = "REUTERS",
       p_attribute = "word",
       registry = system.file(package = "RcppCWB", "extdata", "cwb", "registry"),
